package org.jvending.provisioning.stocking.handler;

import junit.framework.TestCase;
import org.jvending.registry.RepositoryRegistry;
import org.jvending.provisioning.stocking.par.DescriptorFileType;
import org.jvending.provisioning.stocking.StockingException;
import org.easymock.MockControl;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class JadHandlerTest extends TestCase {

    private JadHandler jadHandler;

    private static String baseDir = System.getProperty("basedir");

    public void setUp() {
        RepositoryRegistry repositoryRegistry = RepositoryRegistry.Factory.create();
        try {
            repositoryRegistry.loadFromFile(baseDir + "/src/test/resources/registry-config.xml");
        } catch (IOException e) {
            e.printStackTrace();
            fail("Unable to load the RepositoryRegistry");
        }
        jadHandler = new JadHandler();
        jadHandler.setRepositoryRegistry(repositoryRegistry);
    }

    public void testNullDescriptorType() {
        List files = null;
        try {
            files = jadHandler.getContentFiles(null, new HashMap());
            fail("Did not throw StockingException for null Descriptor");
        } catch (StockingException e) {

        }
    }

    public void testGettingContentFile() {
        String jad = "games.jad";
        String jar = "games.jar";
        String mimeType = null;

        DescriptorFileType descriptorFileType = getMockDescriptorFileType(jad, mimeType);
        Map contentMap = getMockContentMapWithJar(jad, jar);

        try {
            List files = jadHandler.getContentFiles(descriptorFileType, contentMap);
            assertEquals("Wrong number of content items", 1, files.size());
        } catch (StockingException e) {
            fail("Could not obtain ContentFile: Message = " + e.getMessage());
        }
    }

    public void testWithNullJad() {
        String jad = null;
        String jar = "games.jar";
        String mimeType = "text/vnd.sun.j2me.app-descriptor";

        DescriptorFileType descriptorFileType = getMockDescriptorFileType(jad, mimeType);
        Map contentMap = getMockContentMapWithJar(jad, jar);

        try {
            List files = jadHandler.getContentFiles(descriptorFileType, contentMap);
            assertEquals("Wrong number of content items", 0, files.size());
        } catch (StockingException e) {
            fail("Could not obtain ContentFile: Message = " + e.getMessage());
        }
    }

    public void testWithRemoteUriForJar() {
        String jad = "games.jad";
        String jar = "http://jvending.org/games.jar";
        String mimeType = "text/vnd.sun.j2me.app-descriptor";

        DescriptorFileType descriptorFileType = getMockDescriptorFileType(jad, mimeType);
        Map contentMap = new HashMap();
        contentMap.put(jad, ("MIDlet-Jar-URL: "+ jar).getBytes());

        try {
            List files = jadHandler.getContentFiles(descriptorFileType, contentMap);
            assertEquals("Wrong number of content items", 0, files.size());
        } catch (StockingException e) {
            fail("Could not obtain ContentFile: Message = " + e.getMessage());
        }
    }

    public void testWithRemoteUriForJad() {
        String jad = "http://jvending.org/games.jad";
        String jar = null;
        String mimeType = "text/vnd.sun.j2me.app-descriptor";

        DescriptorFileType descriptorFileType = getMockDescriptorFileType(jad, mimeType);
        Map contentMap = new HashMap();
        contentMap.put(jad, ("MIDlet-Jar-URL: "+ jar).getBytes());
        try {
            List files = jadHandler.getContentFiles(descriptorFileType, contentMap);
            assertEquals("Wrong number of content items", 0, files.size());
        } catch (StockingException e) {
            fail("Could not obtain ContentFile: Message = " + e.getMessage());
        }
    }

    private Map getMockContentMapWithJar(String jad, String jar) {
        Map contentMap = new HashMap();
        contentMap.put(jad, ("MIDlet-Jar-URL: "+ jar).getBytes());
        byte[] fakeJarBytes = {10, 12, 13};
        contentMap.put(jar, fakeJarBytes);
        return contentMap;
    }

    private DescriptorFileType getMockDescriptorFileType(String url, String mimeType) {
        MockControl controlDescriptorFileType = MockControl.createControl(DescriptorFileType.class);
        DescriptorFileType descriptorFileType = (DescriptorFileType) controlDescriptorFileType.getMock();

        descriptorFileType.getValue();
        controlDescriptorFileType.setReturnValue(url);
        descriptorFileType.getValue();
        controlDescriptorFileType.setReturnValue(url);

        descriptorFileType.getMimeType();
        controlDescriptorFileType.setReturnValue(mimeType);

        controlDescriptorFileType.replay();
        return descriptorFileType;
    }
}
